package net.wanji.opt.controller;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import io.swagger.annotations.ResponseHeader;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.AbstractRestServer;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.impl.AbstractRestServerImpl;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.service.GreenwaveHistProvider;
import net.wanji.opt.vo.GreenWaveRunStateVO;
import net.wanji.opt.vo.GreenwaveCrossMetricsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 绿波历史数据 接口API
 * </p>
 *  @version 1.0
 * @author fengyi
 * @Date 2024-11-18
 */
@RestController
@Slf4j
@RequestMapping("/greenwave-hist")
@Api(value="GreenwaveHistRestServer", description="绿波历史数据接口", tags = "绿波干线评价")
public class GreenwaveHistRestServer   {

    @Autowired
    private GreenwaveHistProvider greenwaveHistProvider;


    @GetMapping("/findGreenWaveRunState")
    @ApiOperation(httpMethod="GET",value="绿波干线运行状态查询", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenId", value = "绿波ID", required = true, dataType = "Integer",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-05 00:00:00")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenWaveRunStateVO.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenWaveRunState(Integer greenId,@RequestParam(defaultValue ="2024-12-04 00:00:00") String startTime,
                                             @RequestParam(defaultValue ="2024-12-05 00:00:00") String endTime,
                                                @RequestParam(defaultValue ="705,706")  String eventType) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            List<GreenWaveRunStateVO> list = greenwaveHistProvider.findGreenWaveRunState(greenId,null,startTime,endTime,eventType);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }


    @GetMapping("/findStatisticIndexTrend")
    @ApiOperation(httpMethod="GET",value="绿波历史数据-数据指标统计", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenId", value = "绿波ID", required = true, dataType = "String",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-05 00:00:00")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findStatisticIndex(String greenId,@RequestParam(defaultValue ="2024-11-12 00:00:00") String startTime,
                                             @RequestParam(defaultValue ="2024-11-13 00:00:00") String endTime) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            JSONObject list = greenwaveHistProvider.findStatisticIndex(greenId,startTime,endTime,null);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }


    @GetMapping("/findGreenCrossInfo")
    @ApiOperation(httpMethod="GET",value="绿波干线路口信息查询", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenId", value = "绿波ID", required = true, dataType = "String",defaultValue = "1")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenCrossInfo(Integer greenId) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            List<Map<String,Object>> list = greenwaveHistProvider.findGreenCross(greenId);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }

    @GetMapping("/findUnCoordinateDirIndex")
    @ApiOperation(httpMethod="GET",value="绿波非协调方向->指标趋势", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "greenDir", value = "绿波协调方向,多个逗号分隔", required = true, dataType = "String",defaultValue = "5,7"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 23:59:59")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findUnCoordinateDirIndex(String crossId,String greenDir,String startTime,String endTime) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            JSONObject list = greenwaveHistProvider.findUnCoordinateCrossIndex(crossId,greenDir,startTime,endTime);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }


    @GetMapping("/findGreenWaveCrossDirIndex")
    @ApiOperation(httpMethod="GET",value="绿波路口方向级->分粒度指标趋势", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度类型 0：5分钟粒度 1：15分钟粒度 2：30分钟粒度 3：小时粒度", required = true, dataType = "Integer",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 23:59:59")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenWaveCrossDirIndex(String crossId, String startTime, String endTime,String groupType) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            JSONObject list = greenwaveHistProvider.findGreenWaveCrossDirIndex(crossId,startTime,endTime,groupType);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }

    @GetMapping("/findGreenWaveCrossTurnIndex")
    @ApiOperation(httpMethod="GET",value="绿波路口转向级->分粒度指标趋势", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度类型 0：5分钟粒度 1：15分钟粒度 2：30分钟粒度 3：小时粒度", required = true, dataType = "Integer",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-05 00:00:00")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenWaveCrossTurnIndex(String crossId, String startTime, String endTime,String groupType) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            JSONObject list = greenwaveHistProvider.findGreenWaveCrossTurnIndex(crossId,startTime,endTime,groupType);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }


    @GetMapping("/findGreenWaveCrossObjectIndex")
    @ApiOperation(httpMethod="GET",value="路口方向级、转向级、车道级->分粒度指标趋势", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度类型 0：5分钟粒度 1：15分钟粒度 2：30分钟粒度 3：小时粒度 4：天粒度", required = true, dataType = "Integer",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2024-12-05 00:00:00"),
            @ApiImplicitParam(name = "objectType", value = "统计对象类型 1：方向级指标 2：转向级指标 3：车道级指标", required = true, dataType = "Integer",defaultValue = "1"),
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenWaveCrossObjectIndex(String crossId, String startTime, String endTime,String groupType,Integer objectType) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            JSONObject list = greenwaveHistProvider.findGreenWaveCrossLaneIndex(crossId,startTime,endTime,groupType,objectType);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }


    @GetMapping("/findGreenWaveCrossIndex")
    @ApiOperation(httpMethod="GET",value="绿波路口级->分粒度指标趋势", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度类型 1：15分钟粒度 2：30分钟粒度 3：小时粒度", required = true, dataType = "Integer",defaultValue = "1"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-04 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = false, dataType = "String",defaultValue = "2024-12-05 00:00:00")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject findGreenWaveCrossIndex(String crossId, String startTime, String endTime,String groupType) {
        JsonViewObject jsonView = JsonViewObject.newInstance();

        try {
            JSONObject list = greenwaveHistProvider.findGreenWaveCrossIndex(crossId,startTime,endTime,groupType);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }

        return jsonView;
    }



}
